@@ -5,8 +5,8 @@
     :license: Simplified BSD License, see LICENSE.txt for more details."""
 
 from passlib.context import CryptContext
-from flask import Blueprint, current_app, url_for
-from flask_login import LoginManager, UserMixin as LoginUserMixin, make_secure_token
+from flask import Blueprint, current_app, url_for, render_template
+from flask_login import LoginManager, UserMixin as LoginUserMixin
 from flask_user.db_adapters import DBAdapter
 from .db_adapters import SQLAlchemyAdapter
 from . import emails
@@ -19,16 +19,16 @@ from . import views
 from . import signals
 from .translations import get_translations
 
-# Enable the following: from flask.ext.user import current_user
+# Enable the following: from flask_user import current_user
 from flask_login import current_user
 
-# Enable the following: from flask.ext.user import login_required, roles_required
+# Enable the following: from flask_user import login_required, roles_required
 from .decorators import *
-# Enable the following: from flask.ext.user import user_logged_in
+# Enable the following: from flask_user import user_logged_in
 from .signals import *
 
 
-__version__ = '0.6.8'
+__version__ = '0.6.11'
 
 
 def _call_or_get(function_or_property):
@@ -68,6 +68,7 @@ class UserManager(object):
                 username_validator=forms.username_validator,
                 password_validator=forms.password_validator,
                 # View functions
+                render_function=render_template,
                 change_password_view_function=views.change_password,
                 change_username_view_function=views.change_username,
                 confirm_email_view_function=views.confirm_email,
@@ -88,6 +89,7 @@ class UserManager(object):
                 login_manager=LoginManager(),
                 password_crypt_context=None,
                 send_email_function = emails.send_email,
+                make_safe_url_function = views.make_safe_url,
                 token_manager=tokens.TokenManager(),
                 legacy_check_password_hash=None
                 ):
@@ -109,6 +111,7 @@ class UserManager(object):
         self.username_validator = username_validator
         self.password_validator = password_validator
         # View functions
+        self.render_function = render_function
         self.change_password_view_function = change_password_view_function
         self.change_username_view_function = change_username_view_function
         self.confirm_email_view_function = confirm_email_view_function
@@ -130,6 +133,7 @@ class UserManager(object):
         self.token_manager = token_manager
         self.password_crypt_context = password_crypt_context
         self.send_email_function = send_email_function
+        self.make_safe_url_function = make_safe_url_function
         self.legacy_check_password_hash = legacy_check_password_hash
 
         """ Initialize app.user_manager."""
@@ -193,15 +197,6 @@ class UserManager(object):
             #print('load_user_by_id: user_id=', user_id)
             return self.get_user_by_id(user_id)
 
-        # Flask-login calls this function to retrieve a User record by user token.
-        # A token is used to secure the user ID when stored in browser sessions.
-        # See https://flask-login.readthedocs.org/en/latest/#alternative-tokens
-        @self.login_manager.token_loader
-        def load_user_by_token(token):
-            user_id = self.token_manager.decrypt_id(token)
-            #print('load_user_by_token: token=', token, 'user_id=', user_id)
-            return self.get_user_by_id(int(user_id))
-
         self.login_manager.login_view = 'user.login'
         self.login_manager.init_app(app)
 
